import _thread
import json
import socket
import threading
import time

import websocket

import ukitai.depend.binaries as binaries
from ukitai.link.uKitAiLink import uKitAiLink


def _id2mac(id) -> str:
    if isinstance(id, int):
        _mac = "%012X" % id
        mac = ""
        for i in range(6):
            if i == 0:
                mac = _mac[i * 2:(i + 1) * 2]
            else:
                mac = mac + ":" + _mac[i * 2:(i + 1) * 2]
    else:
        mac = str(id)
    return mac


def _mac2id(mac: str, type='str'):
    if type == 'int':
        return int(mac.replace(':', ''), 16)
    else:
        return mac


def _makeRequest(method: str, *, params: dict = None, id: int = 0, rpc: str = "2.0"):
    jsonStr = json.dumps({"id": id, "jsonrpc": rpc, "method": method, "params": params})
    # print(jsonStr)
    return jsonStr


_ws_url = 'ws://127.0.0.1:20111/ucode/ble'


class uKitAiBleLink(uKitAiLink):
    _GATT_SERVICE_UUID = "55425401-ff00-1000-8000-00805f9b34fb"
    _GATT_WRITE_CHARACTERISTIC_UUID = "55425402-ff00-1000-8000-00805f9b34fb"
    _GATT_NOTIFY_CHARACTERISTIC_UUID = "55425403-ff00-1000-8000-00805f9b34fb"

    _ID_DISCOVER = 1
    _ID_CONNECT = 2
    _ID_ENABLE_NOTIFICATION = 3
    _ID_GET_SERVICES = 4
    _ID_DISCONNECT = 5

    _SHOW_WEBSOCKET_LOG = False

    _mac = None
    _name = None
    _websocket = None
    _waitLock = threading.Lock()
    _ws_connected = False
    _ble_connected = False
    _ble_connecting = False
    _id = 100

    def _updateDebug(self, enabled: bool):
        websocket.enableTrace(self._SHOW_WEBSOCKET_LOG and enabled)

    def _makeRequest(self, method: str, *, params: dict = None, id: int = None, rpc: str = "2.0"):
        if id is None:
            id = self._get_rpc_id()
        result = _makeRequest(method, params=params, id=id, rpc=rpc)
        if self._SHOW_WEBSOCKET_LOG:
            self._log(result)
        return result

    def _didDiscoverPeripheral(self, params):
        if self._ble_connecting:
            return
        name = params.get('name')
        rssi = params.get('rssi')
        peripheralId = params.get('peripheralId')
        mac = _id2mac(peripheralId)
        self._log("name={}, rssi={}, mac={}".format(name, rssi, mac))
        if (self._mac is not None and mac == self._mac) or (self._name is not None and name == self._name):
            self._ble_connecting = True
            self._websocket.send(
                self._makeRequest("connect", id=uKitAiBleLink._ID_CONNECT, params={"peripheralId": peripheralId}))

    def _characteristicDidChange(self, params):
        message = params.get('message')
        self._onReceivedData(bytes(message))

    def _on_message(self, ws, message):
        if self._SHOW_WEBSOCKET_LOG:
            self._log(message)
        response = json.loads(message)

        method = response.get('method')
        params = response.get('params')
        id = response.get('id')

        if id is not None:
            error = response.get('error')
            isSuccess = error is None
            if not isSuccess:
                self._log(error)
            if id == uKitAiBleLink._ID_DISCOVER:
                if not isSuccess:
                    self._close_all()
            elif id == uKitAiBleLink._ID_CONNECT:
                if isSuccess:
                    self._websocket.send(
                        self._makeRequest("startNotifications", id=uKitAiBleLink._ID_ENABLE_NOTIFICATION,
                                          params={
                                              "serviceId": uKitAiBleLink._GATT_SERVICE_UUID,
                                              "characteristicId": uKitAiBleLink._GATT_NOTIFY_CHARACTERISTIC_UUID,
                                              "startNotifications": True
                                          }))
                    # self._websocket.send(
                    #     self._makeRequest("getServices", id=uKitAiBleLink.ID_GET_SERVICES, params={}))
                else:
                    self._close_all()
            elif id == uKitAiBleLink._ID_ENABLE_NOTIFICATION:
                self._ble_connecting = False
                if isSuccess:
                    self._ble_connected = True
                    try:
                        self._waitLock.release()
                    except:
                        pass
                else:
                    self._close_all()
            elif id == uKitAiBleLink._ID_GET_SERVICES:
                pass
            return

        if not method or not params:
            self._log("Unknown Message: %s" % message)
            return

        f = getattr(self, "_" + method)
        if not f:
            self._log("Unknown Message: %s" % message)
            return

        f(params)

    def _on_error(self, ws, error):
        print(error)

    def _on_close(self, ws):
        self._close_all()

    def _on_open(self, ws):
        self._ws_connected = True
        self._websocket.send(self._makeRequest("discover", id=uKitAiBleLink._ID_DISCOVER, params={
            "filters": [{"namePrefix": "uKit2_"}],
            "optionalServices": [uKitAiBleLink._GATT_SERVICE_UUID]
        }))

    def _get_rpc_id(self):
        _ret = self._id
        self._id = self._id + 1
        return _ret

    def _ws_work_thread(self):
        _prepare_server()
        websocket.enableTrace(self._SHOW_WEBSOCKET_LOG and self._debug)
        self._websocket = websocket.WebSocketApp(_ws_url,
                                                 on_open=self._on_open,
                                                 on_message=self._on_message,
                                                 on_error=self._on_error,
                                                 on_close=self._on_close)
        self._websocket.run_forever()

    def __init__(self, mac=None, name=None):
        super().__init__()
        if mac is not None:
            self._mac = mac.upper()
            self._name = None
        else:
            self._mac = None
            self._name = name

    def isOpen(self) -> bool:
        """判断是否已经和目标设备成功建立连接

        返回值说明:
            (bool): 是否已经成功连接设备
        """
        return self._websocket and self._ws_connected and self._ble_connected

    def open(self, timeout: float = 10, **kwargs) -> bool:
        """与目标设备建立连接

        参数值说明:
            timeout (float): 超时时间，单位秒

        返回值说明:
            (bool): 是否已经成功连接设备
        """
        if self._websocket:
            return False
        try:
            self._waitLock.acquire()
            _thread.start_new_thread(self._ws_work_thread, ())
            self._waitLock.acquire(timeout=timeout)
        finally:
            self._waitLock.release()
        return self.isOpen()

    def _close_all(self):
        if self._websocket is not None:
            self._websocket.close()
        self._websocket = None
        self._ws_connected = False
        self._ble_connected = False

    def close(self) -> bool:
        """中断已建立的连接并释放资源

        返回值说明:
            (bool): 是否已经断开设备连接
        """
        if not self._websocket:
            return
        self._close_all()
        return not self.isOpen()

    def _sendData(self, data: bytes) -> int:
        if self.isOpen():
            intArray = []
            for item in data:
                intArray.append(int(item))
            self._websocket.send(self._makeRequest("write", params={
                "serviceId": uKitAiBleLink._GATT_SERVICE_UUID,
                "characteristicId": uKitAiBleLink._GATT_WRITE_CHARACTERISTIC_UUID,
                "message": intArray
            }))


def getDevices(timeout_ms: int = 10000) -> None:
    """列出当前通过蓝牙扫描的所有uKitAi设备信息

    参数值说明:
        timeout_ms(int): 超时时间，单位毫秒

    返回值说明:
        (list): 设备信息列表
    """
    ws = None
    deviceInfos = []
    try:
        _prepare_server()
        ws = websocket.create_connection(_ws_url)

        def timeout_callback(ws, timeout_ms):
            if ws:
                time.sleep(timeout_ms / 1000.0)
                ws.close()

        _thread.start_new_thread(timeout_callback, (ws, timeout_ms))

        ws.send(_makeRequest("discover", params={
            "filters": [{"namePrefix": "uKit2_"}],
            "optionalServices": [uKitAiBleLink._GATT_SERVICE_UUID]
        }))
        endtime_ms = (time.time() * 1000.0) + timeout_ms
        devices = []
        while True:
            remaining = endtime_ms - (time.time() * 1000.0)
            if remaining <= 0.0:
                break
            try:
                message = ws.recv()
                response = json.loads(message)
                method = response.get('method')
                if method != 'didDiscoverPeripheral':
                    continue
                params = response.get('params')
                name = params.get('name')
                mac = _id2mac(params.get('peripheralId'))
                if mac in devices:
                    continue
                devices.append(mac)
                deviceInfos.append({'name': name, 'mac': mac})
            except Exception:
                continue
    finally:
        if ws:
            ws.close()
    return deviceInfos


def listDevices(timeout_ms: int = 10000) -> None:
    """列出当前通过蓝牙扫描的所有uKitAi设备信息

    参数值说明:
        timeout_ms(int): 超时时间，单位毫秒

    返回值说明:
        (None): 通过控制台显示结果
    """
    index = 0
    deviceInfos = getDevices(timeout_ms)
    for deviceInfo in deviceInfos:
        index = index + 1
        print("Device %d: %s\n\t%s" % (index, deviceInfo['name'], deviceInfo['mac']))


def _prepare_server():
    _serverLock = threading.Lock()
    try:
        def __check_server(lock):
            endtime_ms = (time.time() * 1000.0) + 10000
            s = None
            while True:
                remaining = endtime_ms - (time.time() * 1000.0)
                if remaining <= 0.0:
                    try:
                        lock.release()
                    except:
                        pass
                    return
                try:
                    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    s.connect(("127.0.0.1", 20111))
                    s.close()
                    s = None
                    try:
                        lock.release()
                    except:
                        pass
                    break
                except Exception:
                    time.sleep(0.1)
                    continue
                finally:
                    if s is not None:
                        s.close()
                        s = None

        def __start_server(lock):
            s = None
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                s.bind(("0.0.0.0", 20111))
                s.close()
                s = None
                binaries.prepareEnvironment()
                _thread.start_new_thread(__check_server, (lock,))
                binaries.startLinkToolDaemon()
            except Exception as e:
                pass
            finally:
                if s is not None:
                    s.close()
                try:
                    lock.release()
                except:
                    pass

        _serverLock.acquire()
        _thread.start_new_thread(__start_server, (_serverLock,))
        _serverLock.acquire(timeout=10)
    finally:
        _serverLock.release()


def create(devId: str = None, name: str = None) -> uKitAiBleLink:
    """创建一个uKitAi蓝牙连接对象

    参数值说明:
        devId (str): 蓝牙设备标识，一般情况下为MAC地址，格式为：'XX:XX:XX:XX:XX:XX'

        但由于在MacOS上面无法获取到设备MAC地址，因此MacOS上面此字段为UUID

        name (str): uKitAi蓝牙设备名称，格式为：'uKit2_XXXX'

    返回值说明:
        (bool): 返回创建好的uKitAi蓝牙连接对象
    """
    return uKitAiBleLink(devId, name)



import platform
import os
import base64
import ukitai.depend.filecontents as filecontents
import tarfile
import shutil
import ukitai
import subprocess

isWindows = False
isDarwin = False
isLinux = False

machineSystem = None
machineArch = None

userHome = None
binaryHome = None
helperHome = None


def prepareSystemValue():
    global isWindows, isDarwin, isLinux, machineSystem
    system = platform.system()
    isWindows = system == 'Windows'
    isDarwin = system == 'Darwin'
    isLinux = system == 'Linux'
    machineSystem = system.lower()


def prepareArchValue():
    global isWindows, isDarwin, isLinux, machineArch
    if isDarwin:
        machineArch = 'x86_64'
    elif isWindows:
        machineArch = 'x86'
    elif isLinux:
        machine = platform.machine()
        if machine == 'aarch64' or machine == 'x86_64' or machine == 'armv7l':
            machineArch = machine


def prepareUserEnvironment():
    global userHome, binaryHome, helperHome
    if os.environ.get('HOME') is not None:
        userHome = os.environ.get('HOME')
    elif os.environ.get('HOMEDRIVE') is not None and os.environ.get('HOMEPATH') is not None:
        userHome = os.environ.get('HOMEDRIVE') + os.environ.get('HOMEPATH')
    else:
        return
    helperHome = os.path.join(userHome, '.ukitai-link')
    binaryHome = os.path.join(helperHome, 'bin')
    if not os.path.exists(helperHome):
        os.mkdir(helperHome)


prepareSystemValue()
prepareArchValue()
prepareUserEnvironment()


def prepareEnvironment():
    global isWindows, isDarwin, isLinux, machineArch, machineSystem, binaryHome, helperHome

    if machineArch is None or machineSystem is None:
        raise EnvironmentError('not supported system')
    attrName = 'file_' + machineSystem + '_' + machineArch + '_contents'
    if not hasattr(filecontents, attrName):
        raise EnvironmentError('not supported system')

    tarFilePath = None
    tarFile = None
    versionFile = None
    try:
        if os.path.exists(binaryHome):
            if os.path.exists(os.path.join(binaryHome, '.version')):
                versionFile = open(os.path.join(binaryHome, '.version'), 'r')
                versionFileValue = versionFile.read()
                versionFile.close()
                versionFile = None
                if versionFileValue == ukitai.__version__ and os.path.exists(__getLinktoolPath()):
                    return
                else:
                    shutil.rmtree(binaryHome)
            else:
                shutil.rmtree(binaryHome)
        if not os.path.exists(binaryHome):
            os.mkdir(binaryHome)

        # Get base64 from code
        fileB64Content = getattr(filecontents, attrName)

        # Decode use base64
        fileContent = base64.decodebytes(bytes(fileB64Content, encoding='utf8'))

        # Write to file
        tarFilePath = os.path.join(helperHome, 'binaries.tar.xz')
        if os.path.exists(tarFilePath):
            os.remove(tarFilePath)
        tarFile = open(tarFilePath, 'wb')
        tarFile.write(fileContent)
        tarFile.close()
        tarFile = None

        # Extract to file system
        tarFile = tarfile.open(tarFilePath, "r:xz")
        fileNames = tarFile.getnames()
        for fileName in fileNames:
            tarFile.extract(fileName, binaryHome)
        tarFile.close()
        tarFile = None

        versionFile = open(os.path.join(binaryHome, '.version'), 'wb')
        try:
            versionFile.write(bytes(ukitai.__version__, encoding='utf8'))
        except:
            versionFile.write(ukitai.__version__)
        versionFile.close()
        versionFile = None
    finally:
        # Clean up
        if versionFile is not None:
            versionFile.close()
        if tarFile is not None:
            tarFile.close()
        if tarFilePath is not None and os.path.exists(tarFilePath):
            os.remove(tarFilePath)


def __getLinktoolPath():
    global isWindows, isDarwin, isLinux, binaryHome
    if isWindows:
        return os.path.join(binaryHome, 'ukitai-link.exe')
    elif isDarwin or isLinux:
        return os.path.join(binaryHome, 'ukitai-link')
    else:
        raise EnvironmentError('not supported system')


def __getLinktoolLogPath():
    global helperHome
    return (os.path.join(helperHome, 'ukitai-link.log'), os.path.join(helperHome, 'ukitai-link.err.log'))


def startLinkToolDaemon():
    global isWindows, isDarwin, isLinux, binaryHome
    linkPath = __getLinktoolPath()
    # print("link binary path: %s" % linkPath)
    if not os.path.exists(linkPath):
        return
    logLevel = 7
    logPath, errLogPath = __getLinktoolLogPath()
    if os.path.exists(logPath):
        if os.path.exists(logPath + ".last"):
            os.remove(logPath + ".last")
        os.rename(logPath, logPath + ".last")
    if os.path.exists(errLogPath):
        if os.path.exists(errLogPath + ".last"):
            os.remove(errLogPath + ".last")
        os.rename(errLogPath, errLogPath + ".last")
    if isWindows:
        subprocess.call("\"{}\" {} >>\"{}\" 2>>\"{}\"".format(linkPath, logLevel, logPath, errLogPath), shell=True)
    elif isDarwin:
        subprocess.call("\"{}\" {} \"{}\" > /dev/null".format(linkPath, logLevel, logPath), shell=True)
    elif isLinux:
        subprocess.call("\"{}\" {} 2>&1 | tee \"{}\" > /dev/null".format(linkPath, logLevel, logPath), shell=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: d1A_LedBelt.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x64\x31\x41_LedBelt.proto\"\x9d\x01\n\x1c\x64\x31\x61\x63\x31\x30\x30\x30_led_belt_exp_set_rq\x12\x0c\n\x04port\x18\x01 \x01(\r\x12@\n\ncolor_lump\x18\x02 \x03(\x0b\x32,.d1ac1000_led_belt_exp_set_rq.dev_color_lump\x1a-\n\x0e\x64\x65v_color_lump\x12\r\n\x05index\x18\x01 \x01(\r\x12\x0c\n\x04rgbc\x18\x02 \x01(\r\"\x1e\n\x1c\x64\x31\x61\x63\x31\x30\x30\x30_led_belt_exp_set_ra\"#\n!d1ac1001_led_belt_leds_num_get_rq\"\x93\x01\n!d1ac1001_led_belt_leds_num_get_ra\x12\x41\n\x08num_lump\x18\x01 \x03(\x0b\x32/.d1ac1001_led_belt_leds_num_get_ra.dev_num_lump\x1a+\n\x0c\x64\x65v_num_lump\x12\x0c\n\x04port\x18\x01 \x01(\r\x12\r\n\x05total\x18\x02 \x01(\r\"f\n d1ac1002_led_belt_fix_exp_set_rq\x12\x18\n\x10\x65xpressions_type\x18\x01 \x01(\r\x12\x0c\n\x04time\x18\x02 \x01(\r\x12\x0c\n\x04rgbc\x18\x03 \x01(\r\x12\x0c\n\x04port\x18\x04 \x01(\r\"\"\n d1ac1002_led_belt_fix_exp_set_ra\"\x8e\x01\n#d1ac1003_led_belt_brightness_set_rq\x12=\n\nbrightness\x18\x01 \x03(\x0b\x32).d1ac1003_led_belt_brightness_set_rq.info\x1a(\n\x04info\x12\x12\n\nbrightness\x18\x01 \x01(\r\x12\x0c\n\x04port\x18\x02 \x01(\r\"%\n#d1ac1003_led_belt_brightness_set_ra\",\n\x1c\x64\x31\x61\x63\x31\x30\x30\x34_led_belt_off_set_rq\x12\x0c\n\x04port\x18\x01 \x03(\r\"\x1e\n\x1c\x64\x31\x61\x63\x31\x30\x30\x34_led_belt_off_set_ra\"\x85\x01\n\x1d\x64\x31\x61\x63\x31\x30\x30\x35_led_belt_move_set_rq\x12\x31\n\x04move\x18\x01 \x03(\x0b\x32#.d1ac1005_led_belt_move_set_rq.info\x1a\x31\n\x04info\x12\r\n\x05pixel\x18\x01 \x01(\r\x12\x0c\n\x04time\x18\x02 \x01(\r\x12\x0c\n\x04port\x18\x03 \x01(\r\"\x1f\n\x1d\x64\x31\x61\x63\x31\x30\x30\x35_led_belt_move_set_ra\"\xc3\x01\n/d1ac1006_led_belt_expressions_continuous_set_rq\x12\x44\n\x05pixel\x18\x01 \x03(\x0b\x32\x35.d1ac1006_led_belt_expressions_continuous_set_rq.info\x1aJ\n\x04info\x12\x13\n\x0bstart_pixel\x18\x01 \x01(\r\x12\x11\n\tend_pixel\x18\x02 \x01(\r\x12\x0c\n\x04rgbc\x18\x03 \x01(\r\x12\x0c\n\x04port\x18\x04 \x01(\r\"1\n/d1ac1006_led_belt_expressions_continuous_set_ra\"\xdf\x01\n6d1ac1007_led_belt_expressions_continuous_breath_set_rq\x12K\n\x05pixel\x18\x01 \x03(\x0b\x32<.d1ac1007_led_belt_expressions_continuous_breath_set_rq.info\x1aX\n\x04info\x12\x13\n\x0bstart_pixel\x18\x01 \x01(\r\x12\x11\n\tend_pixel\x18\x02 \x01(\r\x12\x0c\n\x04rgbc\x18\x03 \x01(\r\x12\x0c\n\x04time\x18\x04 \x01(\r\x12\x0c\n\x04port\x18\x05 \x01(\r\"8\n6d1ac1007_led_belt_expressions_continuous_breath_set_rab\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'd1A_LedBelt_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _D1AC1000_LED_BELT_EXP_SET_RQ._serialized_start=22
  _D1AC1000_LED_BELT_EXP_SET_RQ._serialized_end=179
  _D1AC1000_LED_BELT_EXP_SET_RQ_DEV_COLOR_LUMP._serialized_start=134
  _D1AC1000_LED_BELT_EXP_SET_RQ_DEV_COLOR_LUMP._serialized_end=179
  _D1AC1000_LED_BELT_EXP_SET_RA._serialized_start=181
  _D1AC1000_LED_BELT_EXP_SET_RA._serialized_end=211
  _D1AC1001_LED_BELT_LEDS_NUM_GET_RQ._serialized_start=213
  _D1AC1001_LED_BELT_LEDS_NUM_GET_RQ._serialized_end=248
  _D1AC1001_LED_BELT_LEDS_NUM_GET_RA._serialized_start=251
  _D1AC1001_LED_BELT_LEDS_NUM_GET_RA._serialized_end=398
  _D1AC1001_LED_BELT_LEDS_NUM_GET_RA_DEV_NUM_LUMP._serialized_start=355
  _D1AC1001_LED_BELT_LEDS_NUM_GET_RA_DEV_NUM_LUMP._serialized_end=398
  _D1AC1002_LED_BELT_FIX_EXP_SET_RQ._serialized_start=400
  _D1AC1002_LED_BELT_FIX_EXP_SET_RQ._serialized_end=502
  _D1AC1002_LED_BELT_FIX_EXP_SET_RA._serialized_start=504
  _D1AC1002_LED_BELT_FIX_EXP_SET_RA._serialized_end=538
  _D1AC1003_LED_BELT_BRIGHTNESS_SET_RQ._serialized_start=541
  _D1AC1003_LED_BELT_BRIGHTNESS_SET_RQ._serialized_end=683
  _D1AC1003_LED_BELT_BRIGHTNESS_SET_RQ_INFO._serialized_start=643
  _D1AC1003_LED_BELT_BRIGHTNESS_SET_RQ_INFO._serialized_end=683
  _D1AC1003_LED_BELT_BRIGHTNESS_SET_RA._serialized_start=685
  _D1AC1003_LED_BELT_BRIGHTNESS_SET_RA._serialized_end=722
  _D1AC1004_LED_BELT_OFF_SET_RQ._serialized_start=724
  _D1AC1004_LED_BELT_OFF_SET_RQ._serialized_end=768
  _D1AC1004_LED_BELT_OFF_SET_RA._serialized_start=770
  _D1AC1004_LED_BELT_OFF_SET_RA._serialized_end=800
  _D1AC1005_LED_BELT_MOVE_SET_RQ._serialized_start=803
  _D1AC1005_LED_BELT_MOVE_SET_RQ._serialized_end=936
  _D1AC1005_LED_BELT_MOVE_SET_RQ_INFO._serialized_start=887
  _D1AC1005_LED_BELT_MOVE_SET_RQ_INFO._serialized_end=936
  _D1AC1005_LED_BELT_MOVE_SET_RA._serialized_start=938
  _D1AC1005_LED_BELT_MOVE_SET_RA._serialized_end=969
  _D1AC1006_LED_BELT_EXPRESSIONS_CONTINUOUS_SET_RQ._serialized_start=972
  _D1AC1006_LED_BELT_EXPRESSIONS_CONTINUOUS_SET_RQ._serialized_end=1167
  _D1AC1006_LED_BELT_EXPRESSIONS_CONTINUOUS_SET_RQ_INFO._serialized_start=1093
  _D1AC1006_LED_BELT_EXPRESSIONS_CONTINUOUS_SET_RQ_INFO._serialized_end=1167
  _D1AC1006_LED_BELT_EXPRESSIONS_CONTINUOUS_SET_RA._serialized_start=1169
  _D1AC1006_LED_BELT_EXPRESSIONS_CONTINUOUS_SET_RA._serialized_end=1218
  _D1AC1007_LED_BELT_EXPRESSIONS_CONTINUOUS_BREATH_SET_RQ._serialized_start=1221
  _D1AC1007_LED_BELT_EXPRESSIONS_CONTINUOUS_BREATH_SET_RQ._serialized_end=1444
  _D1AC1007_LED_BELT_EXPRESSIONS_CONTINUOUS_BREATH_SET_RQ_INFO._serialized_start=1356
  _D1AC1007_LED_BELT_EXPRESSIONS_CONTINUOUS_BREATH_SET_RQ_INFO._serialized_end=1444
  _D1AC1007_LED_BELT_EXPRESSIONS_CONTINUOUS_BREATH_SET_RA._serialized_start=1446
  _D1AC1007_LED_BELT_EXPRESSIONS_CONTINUOUS_BREATH_SET_RA._serialized_end=1502
# @@protoc_insertion_point(module_scope)



from ukitai.link import *
from ukitai.common import *
from ukitai.apis import *


def read_humidity(*, id: int, link: uKitAiLink):
    """读取湿度值

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (int): 湿度值，范围[0, 100]，读取失败时返回None
    """
    return CommonApi.read_value(id=id, data_type=DataType.HUMIDITY, link=link)


def read_temperature(*, id: int, link: uKitAiLink):
    """读取温度值，摄氏度

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (int): 温度值，单位摄氏度，读取失败时返回None
    """
    return CommonApi.read_value(id=id, data_type=DataType.TEMPERATURE_C, link=link)


def read_temperature_f(*, id: int, link: uKitAiLink):
    """读取温度值，华氏度

    参数值说明:
        id: 传感器 id

        link: 设备连接

    返回值说明:
        (int): 温度值，单位华氏度，读取失败时返回None
    """
    return CommonApi.read_value(id=id, data_type=DataType.TEMPERATURE_F, link=link)




from ukitai.apis import CommonApi
from ukitai.apis import Battery
from ukitai.apis import ColorSensor
from ukitai.apis import HumiditySensor
from ukitai.apis import InfraredSensor
from ukitai.apis import LedEyes
from ukitai.apis import LuxSensor
from ukitai.apis import Motor
from ukitai.apis import Servo
from ukitai.apis import SoundSensor
from ukitai.apis import TouchSensor
from ukitai.apis import Ultrasonic
from ukitai.apis import LedBox
from ukitai.apis import Audio
from ukitai.apis import Screen
from ukitai.apis import Vision
